#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import pathlib
from typing import Dict

from delivery.destinations.interfaces import (
    DeliveryContextIF,
    Destination,
    DestinationTempFile,
)
from delivery.products import Calibration

from test_api import dest_dir


class FakeDestination(Destination):
    def __init__(self):
        super().__init__(None)
        self.files_added = {}

    def add_file(self, file: pathlib.Path, relative_path: str):
        self.files_added[relative_path] = file

    def create_file(self, relative_path: str) -> DestinationTempFile:
        raise NotImplementedError

    def results(self) -> Dict:
        return {}


def test_web_calibration(tmpdir: pathlib.Path, dest_dir: pathlib.Path, resource_path_root):
    calibration = Calibration(resource_path_root / "calibrations" / "foo")

    # first, let's make sure we parsed the metadata properly
    project_code, pipeline_id = calibration.parse_metadata()
    assert project_code == "21A-409"
    assert pipeline_id == "59311.953874212966"

    # OK now let's make sure that we're delivering properly
    destination = FakeDestination()
    calibration.deliver_to(destination)

    expected_files = set(
        [
            "21A-409/calibration_pipeline.59311.953874212966/pipeline-20210409T164741/.keep",
            "21A-409/calibration_pipeline.59311.953874212966/unknown.hifv_cal.pipeline_manifest.xml",
            "21A-409/calibration_pipeline.59311.953874212966/casa_commands.log",
            "21A-409/calibration_pipeline.59311.953874212966/flux.csv",
            "21A-409/calibration_pipeline.59311.953874212966/unknown.hifv_cal.pprequest.xml",
            "21A-409/calibration_pipeline.59311.953874212966/21A-409.sb39530507.eb39564922.59311.953874212966.ms.calapply.txt",
            "21A-409/calibration_pipeline.59311.953874212966/casa_pipescript.py",
            "21A-409/calibration_pipeline.59311.953874212966/casa_piperestorescript.py",
            "21A-409/calibration_pipeline.59311.953874212966/PPR.xml",
            "21A-409/calibration_pipeline.59311.953874212966/pipeline_aquareport.xml",
            "21A-409/calibration_pipeline.59311.953874212966/weblog.tgz",
        ]
    )
    actual_files = destination.files_added.keys()

    # did we get all the expected files?
    for file in expected_files:
        assert file in actual_files

    # did we not get any unexpected files?
    for file in actual_files:
        assert file in expected_files
